package net.wanji.utc.config;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * @author duanruiming
 * @date 2023/02/14 10:17
 */
@Configuration
public class ThreadPoolConfig {
    @Value("${threadPoolConfig.threadPoolName}")
    private String threadPoolName;
    @Value("${threadPoolConfig.coreSize}")
    private int coreSize;
    @Value("${threadPoolConfig.maxSize}")
    private int maxSize;
    @Value("${threadPoolConfig.queueCapacity}")
    private int queueCapacity;
    @Value("${threadPoolConfig.keepAliveTime}")
    private int keepAliveTime;
    @Value("${threadPoolConfig.allowCoreTimeOut}")
    private boolean allowCoreTimeOut;

    @Bean
    public ThreadPoolTaskExecutor threadPoolExecutor() {
        ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
        ThreadFactory build = new ThreadFactoryBuilder().setNameFormat(threadPoolName).build();
        executor.setThreadFactory(build);
        executor.setCorePoolSize(coreSize);
        executor.setMaxPoolSize(maxSize);
        executor.setQueueCapacity(queueCapacity);
        executor.setKeepAliveSeconds(keepAliveTime);
        executor.setAllowCoreThreadTimeOut(allowCoreTimeOut);
        executor.setRejectedExecutionHandler(new ThreadPoolExecutor.DiscardOldestPolicy());
        executor.initialize();
        return executor;
    }


}
