package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.constant.RedisKeyConst;
import net.wanji.utc.common.typeenum.HkControlModeEnum;
import net.wanji.utc.po.SignalStatusLogPO;
import net.wanji.utc.service.runninginfo.HkRunningStatusService;
import net.wanji.utc.util.PathUtil;
import net.wanji.utc.util.RedisUtil;
import net.wanji.utc.vo.LightsStatusVO;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/21 10:05
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class HkRunningStatusServiceImpl implements HkRunningStatusService {
    private final ArtemisConfig artemisConfig;
    private final RedisUtil redisUtil;

    @Override
    public List<SignalStatusLogPO> getHkRunningStatus(List<BaseCrossInfo> baseCrossInfoList) {
        List<SignalStatusLogPO> result = new ArrayList<>();
        SignalStatusLogPO po;
        try {
            Map<String, String> runningPath = PathUtil.getPathMapByApiCode("getWorkingStateBatch");
            for (BaseCrossInfo baseCrossInfo : baseCrossInfoList) {
                JSONArray jsonBody = new JSONArray();
                jsonBody.add(baseCrossInfo.getCode());
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, runningPath, jsonBody.toJSONString(),
                        null, null, "application/json", null);
                JSONObject object = JSON.parseObject(strResult);
                Object o = object.get("code");
                // 多信号机情况下，跳过返回信息不成功的获取返回消息成功的信号机数据
                if (!Objects.equals(String.valueOf(o), "0")) {
                    continue;
                }
                if (Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
                    JSONArray data = object.getJSONArray("data");
                    for (int i = 0; i < data.size(); i++) {
                        JSONObject obj =  data.getJSONObject(i);
                        po = new SignalStatusLogPO();
                        po.setCrossId(baseCrossInfo.getCrossId());
                        po.setSignalId(baseCrossInfo.getCode());
                        po.setStatus(transferStatus(obj.getInteger("state")));
                        // 告警信息
                        Integer hasFault = obj.getInteger("hasFault");
//                        if (hasFault == 0) {
                            po.setFaultType(hasFault);
//                        }
                        // 从Redis中查询控制类型
                        String lightsStatusVOStr = redisUtil.getHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.LIGHTS_STATUS,
                                baseCrossInfo.getManufacturerCode() + Constants.SEPARATOR_UNDER_LINE + po.getSignalId());
                        LightsStatusVO lightsStatusVO = JSON.parseObject(lightsStatusVOStr, LightsStatusVO.class);
                        Integer controlType = 1;
                        if (Objects.nonNull(lightsStatusVO)) {
                            String runMode = lightsStatusVO.getRunMode();
                            int runModeInt = Integer.parseInt(runMode);
                            controlType = HkControlModeEnum.getCodeByHkCode(runModeInt);
                        }
                        po.setControlType(controlType);
                        result.add(po);
                    }
                }
            }
        } catch(Exception e){
            log.error("系统繁忙，服务器端内部错误！", e);
        }
        return result;
    }

    /**
     * 转换海康信号机运行状态，0 离线 1 在线
     * @param state  海康信号机状态定义 0 在线 1 离线
     * @return
     */
    private int transferStatus(int state) {
        return 0 == state ? 1 : 0;
    }
}
