package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.baseentity.BaseCrossInfo;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.constant.RedisKeyConst;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.mapper.SignalStatusLogMapper;
import net.wanji.utc.po.CrossInfoPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.po.SignalStatusLogPO;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.service.runninginfo.HkRunningStatusService;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.service.runninginfo.WanJiCommonRunningStatusService;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.util.RedisUtil;
import net.wanji.utc.vo.LightsStatusVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @date 2022/11/21 9:11
 */
@RequiredArgsConstructor
@Service
public class SignalStatusServiceImpl implements SignalStatusService {
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final CrossInfoMapper crossInfoMapper;
    private final HkRunningStatusService hkRunningStatusService;
    private final HkLightsStatusService hkLightsStatusService;
    private final RedisUtil redisUtil;
    private final SignalStatusLogMapper signalStatusLogMapper;
    private final WanJiCommonRunningStatusService wanJiRunningStatusService;

    @Override
    @Transactional
    public List<SignalStatusLogPO> runningStatusAlarm() {
        List<SignalStatusLogPO> resList = new ArrayList<>();
        BasicEnum.ManufacturerEnum[] manufacturerEnums = BasicEnum.ManufacturerEnum.values();
        for (BasicEnum.ManufacturerEnum manufacturerEnum : manufacturerEnums) {
            List<BaseCrossInfo> baseCrossInfoList = getBaseCrossInfoList(manufacturerEnum);
            if (baseCrossInfoList == null) {
                continue;
            }
            List<SignalStatusLogPO> signalStatusLogPOList = null;
            if (manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
                // 海康
                // 3.2.5信号机运行状态和告警信息
                signalStatusLogPOList = hkRunningStatusService.getHkRunningStatus(baseCrossInfoList);
            } else {
                // todo 其他厂商
                List<String> crossIdList = baseCrossInfoList.stream().map(BaseCrossInfo::getCrossId).collect(Collectors.toList());
                signalStatusLogPOList = wanJiRunningStatusService.runningStatus(crossIdList);
            }
            // 更新Redis
            if (null != signalStatusLogPOList) {
                for (SignalStatusLogPO signalStatusLogPO : signalStatusLogPOList) {
                    String field = manufacturerEnum.getCode() + Constants.SEPARATOR_UNDER_LINE +
                            signalStatusLogPO.getSignalId();
                    redisUtil.setHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.TELESEME_STATUS, field,
                            signalStatusLogPO.getStatus() + "");
                }
                // 更新日志数据库
                if (ListUtil.isNotEmpty(signalStatusLogPOList)) {
                    signalStatusLogMapper.insertBatch(signalStatusLogPOList);
                    resList.addAll(signalStatusLogPOList);
                }
            }
        }
        return resList;
    }

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) {
        List<SignalStatusLogPO> signalStatusLogPOList = new ArrayList<>();
        return signalStatusLogPOList;
    }

    @Override
    public List<LightsStatusVO> lightStatus(String crossId) {
        List<LightsStatusVO> lightsStatusVOList = new ArrayList<>();
        String manufacturerCode = getManufacturerCode(crossId);
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            lightsStatusVOList.addAll(hkLightsStatusService.getHkLightsStatus(crossId));
        } else {
            // 默认万集
            lightsStatusVOList.addAll(wanJiRunningStatusService.lightsStatus(Arrays.asList(crossId)));
        }
        return lightsStatusVOList;
    }

    private String getManufacturerCode(String crossId) {
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        if (Objects.nonNull(crossInfoPO)) {
            Integer manufacturerId = crossInfoPO.getManufacturerId();
            ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
            if (Objects.nonNull(manufacturerInfoPO)) {
                return manufacturerInfoPO.getCode();
            }
        }
        return "WJ";
    }

    @Override
    @Transactional
    public List<LightsStatusVO> lightStatus() {
        List<LightsStatusVO> resList = new ArrayList<>();
        BasicEnum.ManufacturerEnum[] manufacturerEnums = BasicEnum.ManufacturerEnum.values();
        for (BasicEnum.ManufacturerEnum manufacturerEnum : manufacturerEnums) {
            List<BaseCrossInfo> baseCrossInfoList = getBaseCrossInfoList(manufacturerEnum);
            if (baseCrossInfoList == null) {
                continue;
            }
            List<LightsStatusVO> lightsStatusVOList = null;
            if (manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
                // 海康
                // 3.2.4灯态
                lightsStatusVOList = hkLightsStatusService.getHkLightsStatus(baseCrossInfoList);
            } else {
                // todo 其他厂商
                List<String> crossIdList = baseCrossInfoList.stream().map(BaseCrossInfo::getCrossId).collect(Collectors.toList());
                lightsStatusVOList = wanJiRunningStatusService.lightsStatus(crossIdList);
            }
            // 更新Redis
            if (lightsStatusVOList != null) {
                for (LightsStatusVO lightsStatusVO : lightsStatusVOList) {
                    String field = manufacturerEnum.getCode() + Constants.SEPARATOR_UNDER_LINE +
                            lightsStatusVO.getCode();
                    redisUtil.setHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.LIGHTS_STATUS, field,
                            JSON.toJSONString(lightsStatusVO));
                }
                resList.addAll(lightsStatusVOList);
            }
        }
        return resList;
    }

    private List<BaseCrossInfo> getBaseCrossInfoList(BasicEnum.ManufacturerEnum manufacturerEnum) {
        // 查询路口信息
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(manufacturerEnum.getCode());
        Integer manufacturerId = manufacturerInfoPO.getId();
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(manufacturerId);
        if (ListUtil.isEmpty(crossInfoPOList)) {
            return null;
        }
        // 信号机接口入参
        List<BaseCrossInfo> baseCrossInfoList = new ArrayList<>();
        for (CrossInfoPO infoPO : crossInfoPOList) {
            BaseCrossInfo baseCrossInfo = new BaseCrossInfo();
            baseCrossInfo.setCode(infoPO.getCode());
            baseCrossInfo.setCrossId(infoPO.getId());
            baseCrossInfo.setManufacturerCode(manufacturerEnum.getCode());
            baseCrossInfoList.add(baseCrossInfo);
        }
        return baseCrossInfoList;
    }
}
