package net.wanji.utc.service.staticinfo.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.mapper.CrossInfoMapper;
import net.wanji.utc.mapper.CrossPlanMapper;
import net.wanji.utc.mapper.CrossSchedulesMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.CrossSchedulesPO;
import net.wanji.utc.po.ManufacturerInfoPO;
import net.wanji.utc.po.hk.response.DatePO;
import net.wanji.utc.po.hk.response.HKResponse;
import net.wanji.utc.po.hk.response.ScheduleResponseListPO;
import net.wanji.utc.po.hk.response.ScheduleResponsePO;
import net.wanji.utc.service.staticinfo.HkCrossSchedulesService;
import net.wanji.utc.util.PathUtil;
import net.wanji.utc.util.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @author Kent HAN
 * @date 2022/11/18 17:20
 */
@Service
@RequiredArgsConstructor
public class HkCrossSchedulesServiceImpl implements HkCrossSchedulesService {
    private final ArtemisConfig artemisConfig;
    private final CrossInfoMapper crossInfoMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final ManufacturerInfoMapper manufacturerInfoMapper;

    @Override
    public void crossSchedules(List<String> crossIdList) throws Exception {
        // 请求3.1.9接口，获取时间表
        Map<String, String> path = PathUtil.getPathMapByApiCode("uploadDate");
        List<String> crossCodeList = crossInfoMapper.selectCrossCodesByIds(crossIdList);
        String body = JSONArray.toJSONString(crossCodeList);
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null, null, "application/json", null);
        HKResponse hkResponse = JSONObject.parseObject(responseStr, HKResponse.class);
        if (Objects.nonNull(hkResponse) && Objects.equals(Constants.HK_SUCCESS_STRING_CODE, hkResponse.getCode())) {
            String dataStr = String.valueOf(hkResponse.getData());
            if (StringUtils.isNotBlank(dataStr)) {
                List<ScheduleResponseListPO> scheduleResponseListPOS = JSONObject.parseObject(dataStr, new TypeReference<List<ScheduleResponseListPO>>() {});
                for (ScheduleResponseListPO scheduleResponseListPO : scheduleResponseListPOS) {
                    String crossCode = scheduleResponseListPO.getCrossCode();
                    List<ScheduleResponsePO> schedules = scheduleResponseListPO.getSchedules();
                    List<CrossSchedulesPO> crossSchedulesPOList = new ArrayList<>();
                    if (!CollectionUtils.isEmpty(schedules)) {
                        for (ScheduleResponsePO scheduleResponsePO : schedules) {
                            String code = BasicEnum.ManufacturerEnum.HK.getCode();
                            ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(code);
                            Integer manufacturerId = manufacturerInfoPO.getId();
                            String crossId = crossInfoMapper.selectIdByCodeAndManufacturerId(crossCode, manufacturerId);
                            List<Integer> weeks = scheduleResponsePO.getWeeks();
                            String desc = scheduleResponsePO.getDesc();
                            Integer planNo = scheduleResponsePO.getPlanNo();
                            Integer planId = crossPlanMapper.selectIdByNo(crossId, planNo);
                            for (int k = 0; k < weeks.size(); k++) {
                                Integer week = weeks.get(k);
                                CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                                crossSchedulesPO.setScheduleNo(scheduleResponsePO.getId());
                                crossSchedulesPO.setName(desc);
                                crossSchedulesPO.setCrossId(crossId);
                                crossSchedulesPO.setPlanId(planId);
                                crossSchedulesPO.setWeek(week);
                                if (week != 0) {
                                    crossSchedulesPOList.add(crossSchedulesPO);
                                } else {
                                    // 特殊日期
                                    List<DatePO> dates = scheduleResponsePO.getDates();
                                    for (int m = 0; m < dates.size(); m++) {
                                        DatePO date = dates.get(m);
                                        List<Integer> days = date.getDays();
                                        for (int n = 0; n < days.size(); n++) {
                                            Integer day = days.get(n);
                                            Date specialDate = toDate(date, day);
                                            crossSchedulesPO.setSpecialDate(specialDate);
                                            crossSchedulesPOList.add(crossSchedulesPO);
                                        }
                                    }
                                }
                            }
                            // 插入数据库
                            crossSchedulesMapper.deleteByCrossId(crossId);
                            crossSchedulesMapper.insertBatch(crossSchedulesPOList);
                        }
                    }
                }
            }
        }
    }

    private Date toDate(DatePO date, Integer dayInt) {
        String year = String.valueOf(date.getYear());
        String month = String.valueOf(date.getMonth());
        String day = dayInt.toString();
        if (month.length() == 1) {
            month = "0" + month;
        }
        if (day.length() == 1) {
            day = "0" + day;
        }
        String dateStr = year + "-" + month + "-" + day;
        Date parse = DateUtil.parse(dateStr);
        return parse;
    }
}
