package net.wanji.utc.service.staticinfo.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.utc.common.Result;
import net.wanji.utc.mapper.*;
import net.wanji.utc.po.*;
import net.wanji.utc.service.staticinfo.WanJiCommonStaticInfoService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.vo.PlanSectionVO;
import net.wanji.utc.vo.SchemePhaseLightsVO;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/09 9:21
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class WanJiCommonStaticInfoServiceImpl implements WanJiCommonStaticInfoService {

    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;


    @Override
    public Result<SchemePhaseLightsVO> schemePhaseLights(String crossId, String signalId) {
        Map<String, String> commonParamMap = getCommonParamMap(crossId, signalId);
        JsonViewObject jsonViewObject = rpcService(httpRestUtil, rpcUri, null, JSONObject.toJSONString(commonParamMap));
        SchemePhaseLightsVO content = (SchemePhaseLightsVO) jsonViewObject.getContent();
        if (Objects.nonNull(content)) {
            List<CrossSchemePO> crossSchemeList = content.getCrossSchemeList();
            crossSchemeList.forEach(crossSchemePO -> {
                crossSchemeMapper.deleteOne(crossId, crossSchemePO.getSchemeNo());
                crossSchemeMapper.insertOne(crossSchemePO);
            });
            List<CrossPhasePO> crossPhaseList = content.getCrossPhaseList();
            crossPhaseList.forEach(crossPhasePO -> {
                crossPhaseMapper.deleteBatch(crossId, crossPhasePO.getPlanId());
                crossPhaseMapper.insertOne(crossPhasePO);
            });

            List<CrossLightsPO> crossLightsList = content.getCrossLightsList();
            crossLightsList.forEach(crossLightsPO -> {
                crossLightsMapper.deleteOne(crossId, crossLightsPO.getName());
                crossLightsMapper.insertOne(crossLightsPO);
            });
            List<CrossPhaseLightsPO> crossPhaseLightsPOList = content.getCrossPhaseLightsPOList();
            if (!CollectionUtils.isEmpty(crossPhaseLightsPOList)) {
                crossPhaseLightsMapper.deleteByCrossId(crossPhaseLightsPOList.get(0).getCrossId());
                crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOList);
            }
        }
        return Result.success(content);
    }

    @Override
    public Result<PlanSectionVO> planSection(String crossId, String signalId, Integer planNo) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("crossId", crossId);
        paramMap.put("planNo", String.valueOf(planNo));
        paramMap.put("signalId", signalId);
        JsonViewObject jsonViewObject = rpcService(httpRestUtil, rpcUri, null, JSONObject.toJSONString(paramMap));
        PlanSectionVO content = (PlanSectionVO) jsonViewObject.getContent();
        if (Objects.nonNull(content)) {
            if (Objects.nonNull(content)) {
                List<CrossPlanPO> crossPlanPOList = content.getCrossPlanPOList();
                crossPlanPOList.forEach(crossPlanPO -> {
                    crossPlanMapper.deleteOne(crossId, Integer.parseInt(crossPlanPO.getPlanNo()));
                    crossPlanMapper.insertOne(crossPlanPO);
                });

                List<CrossSectionPO> crossSectionPOList = content.getCrossSectionPOList();
                crossSectionPOList.forEach(crossSectionPO -> {
                    crossSectionMapper.deleteOne(crossId, Integer.parseInt(crossSectionPO.getSectionNo()), null);
                    crossSectionMapper.insertOne(crossSectionPO);
                });
            }
        }
        return Result.success(content);
    }

    @Override
    public Result<List<CrossSchedulesPO>> schedules(String crossId, String signalId) {
        Map<String, String> commonParamMap = getCommonParamMap(crossId, signalId);
        JsonViewObject jsonViewObject = rpcService(httpRestUtil, rpcUri, null, JSONObject.toJSONString(commonParamMap));
        List<CrossSchedulesPO> content = (List<CrossSchedulesPO>) jsonViewObject.getContent();
        if (Objects.nonNull(content)) {
            if (!CollectionUtils.isEmpty(content)) {
                content.forEach(crossSchedulesPO -> {
                    crossSchedulesMapper.deleteByCrossId(crossId);
                    crossSchedulesMapper.insertBatch(content);
                });
            }
        }
        return Result.success(content);
    }

    public static Map<String, String> getCommonParamMap(String crossId, String signalId) {
        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("crossId", crossId);
        paramMap.put("signalId", signalId);
        return paramMap;
    }

    @Override
    public JsonViewObject rpcService(HttpRestUtil httpRestUtil, String url, Map<String, String> headers, String param) {
        return WanJiCommonStaticInfoService.super.rpcService(httpRestUtil, url, headers, param);
    }
}
