package net.wanji.utc.service.systemadmin.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import net.wanji.utc.common.typeenum.ManufacturerApiInfoTypeEnum;
import net.wanji.utc.dto.systemadmin.ManufacturerApiInfoInsertOrUpdateDTO;
import net.wanji.utc.mapper.ManufacturerApiInfoMapper;
import net.wanji.utc.mapper.ManufacturerInfoMapper;
import net.wanji.utc.po.ManufacturerApiInfoPO;
import net.wanji.utc.service.systemadmin.ManufacturerApiInfoService;
import net.wanji.utc.util.PageUtils;
import net.wanji.utc.dto.systemadmin.DeleteByIntegerIdListDTO;
import net.wanji.utc.dto.systemadmin.ManufacturerApiInfoListDTO;
import net.wanji.utc.vo.systemadmin.ManufacturerApiInfoVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author Kent HAN
 * @date 2022/11/24 14:10
 */
@Service
public class ManufacturerApiInfoServiceImpl implements ManufacturerApiInfoService {
    @Autowired
    ManufacturerApiInfoMapper manufacturerApiInfoMapper;
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;

    @Override
    public PageInfo<ManufacturerApiInfoVO> list(ManufacturerApiInfoListDTO manufacturerApiInfoListDTO) {
        Integer pageNum = manufacturerApiInfoListDTO.getPageNum();
        Integer pageSize = manufacturerApiInfoListDTO.getPageSize();

        String name = manufacturerApiInfoListDTO.getName();

        String typeStr = manufacturerApiInfoListDTO.getTypeStr();
        Integer typeCode = null;
        if (typeStr != null) {
            typeCode = ManufacturerApiInfoTypeEnum.getCodeByMsg(typeStr);
        }

        String manufacturerNick = manufacturerApiInfoListDTO.getManufacturerNick();
        Integer manufacturerId = null;
        if (manufacturerNick != null) {
            manufacturerId = manufacturerInfoMapper.selectIdByNick(manufacturerNick);
        }

        PageHelper.startPage(pageNum, pageSize);
        List<ManufacturerApiInfoPO> manufacturerApiInfoPOList = manufacturerApiInfoMapper.selectByOptionals(
                name, typeCode, manufacturerId);
        PageInfo<ManufacturerApiInfoPO> manufacturerApiInfoPOPageInfo = new PageInfo<>(manufacturerApiInfoPOList);
        // 转换为VO类型的PageInfo对象
        PageInfo<ManufacturerApiInfoVO> manufacturerApiInfoOutVOPageInfo =
                PageUtils.PageInfo2PageInfoVo(manufacturerApiInfoPOPageInfo);
        for (ManufacturerApiInfoPO manufacturerApiInfoPO : manufacturerApiInfoPOList) {
            ManufacturerApiInfoVO manufacturerApiInfoVO = new ManufacturerApiInfoVO();
            BeanUtils.copyProperties(manufacturerApiInfoPO, manufacturerApiInfoVO);
            Integer type = manufacturerApiInfoPO.getType();
            String apiType = ManufacturerApiInfoTypeEnum.getMsgByCode(type);
            manufacturerApiInfoVO.setApiType(apiType);
            // 转换厂商简称
            Integer manufacturerId1 = manufacturerApiInfoPO.getManufacturerId();
            String manufacturerNick1 = manufacturerInfoMapper.selectNickById(manufacturerId1);
            manufacturerApiInfoVO.setManufacturerNick(manufacturerNick1);
            manufacturerApiInfoOutVOPageInfo.getList().add(manufacturerApiInfoVO);
        }

        return manufacturerApiInfoOutVOPageInfo;
    }

    @Override
    @Transactional
    public void insertOrUpdate(ManufacturerApiInfoInsertOrUpdateDTO inVO) {
        ManufacturerApiInfoPO manufacturerApiInfoPO = new ManufacturerApiInfoPO();
        BeanUtils.copyProperties(inVO, manufacturerApiInfoPO);
        String typeStr = inVO.getApiType();
        Integer typeCode = ManufacturerApiInfoTypeEnum.getCodeByMsg(typeStr);
        String manufacturerNick = inVO.getManufacturerNick();
        Integer manufacturerId = manufacturerInfoMapper.selectIdByNick(manufacturerNick);
        manufacturerApiInfoPO.setType(typeCode);
        manufacturerApiInfoPO.setManufacturerId(manufacturerId);

        Integer id = inVO.getId();
        if (id == null || id == 0) {
            // 添加
            manufacturerApiInfoMapper.insertOne(manufacturerApiInfoPO);
        } else {
            // 修改
            manufacturerApiInfoMapper.updateOne(manufacturerApiInfoPO);
        }
    }

    @Override
    @Transactional
    public void delete(DeleteByIntegerIdListDTO inVO) {
        List<Integer> ids = inVO.getIds();
        manufacturerApiInfoMapper.deleteBatch(ids);
    }
}
