package net.wanji.utc.vo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.Pattern;
import java.util.List;

/**
 * 下发相位参数VO
 *
 * @author wuxiaokai
 * @date 2022/12/9 11:08:51
 */
@Setter
@Getter
@ApiModel(value = "下发相位参数输入参数", description = "下发相位参数输入参数")
public class PhaseTimingSendVO {

    @ApiModelProperty(value = "路口编号", notes = "路口编号")
    @NotBlank(message = "路口编号不可为空")
    @Pattern(regexp = "^[A-Za-z0-9]{11}$", message = "路口编号只能包含英文、数字，必须11个字符")
    private String crossCode;

    @ApiModelProperty(value = "需要下发的相位列表", notes = "需要下发的相位列表")
    @NotEmpty(message = "相位列表不可为空")
    private List<Phase> phaseList;

    /**
     * 相位
     */
    @Setter
    @Getter
    public static class Phase {

        /**
         * 相位号
         */
        @ApiModelProperty(value = "相位号", notes = "相位号")
        private Integer phaseNo;
        /**
         * 相位描述
         */
        @ApiModelProperty(value = "相位描述", notes = "相位描述")
        private String desc;
        /**
         * 黄灯
         */
        @ApiModelProperty(value = "黄灯", notes = "黄灯")
        private Integer yellow;
        /**
         * 全红
         */
        @ApiModelProperty(value = "全红", notes = "全红")
        private Integer allred;
        /**
         * 绿闪
         */
        @ApiModelProperty(value = "绿闪", notes = "绿闪")
        private String greenFlash;
        /**
         * 红闪
         */
        @ApiModelProperty(value = "红闪", notes = "红闪")
        private String redFlash;
        /**
         * 最小绿
         */
        @ApiModelProperty(value = "最小绿", notes = "最小绿")
        private String minGreen;
        /**
         * 最大绿
         */
        @ApiModelProperty(value = "最大绿", notes = "最大绿")
        private String maxGreen;
        /**
         * 相位关联的车道列表
         */
        @ApiModelProperty(value = "相位关联的车道列表", notes = "相位关联的车道列表")
        private List<Lane> lanes;

        @Setter
        @Getter
        public static class Lane {
            /**
             * 车道号
             */
            @ApiModelProperty(value = "车道号", notes = "车道号")
            private Integer laneNo;
            /**
             * 方向
             */
            @ApiModelProperty(value = "方向", notes = "方向")
            private Integer direction;
            /**
             * 转向
             */
            @ApiModelProperty(value = "转向", notes = "转向")
            private Integer turn;
        }

    }
}
